﻿Class WavePlayer

    ' =====================================================================================
    '  PRIVATE VARS AND FUNCTIONS
    ' =====================================================================================
    Private m_SamplesPerSec As Int32 = 11025 '8000 11025 22050 44100
    Private m_Channels As Int32 = 2
    Private m_NumBuffers As Int32 = 5
    Private m_LenBuffers As Int32 = 256
    Private m_OutputDeviceIndex As Int32 = -1
    Private m_Playing As Boolean = False
    Private AOUT As AudioOut = New AudioOut

    Private m_DriveVolume As Single = 0.001
    Private m_OutVolume As Single = Short.MaxValue * 0.2
    Private m_Fmin As Single = 200
    Private m_Fmax As Single = 1500
    Private m_WaveType As WaveTypes = WaveTypes.Sine

    Private m_WaveTable() As Single
    Private m_vol1(-1) As Single
    Private m_vol2(-1) As Single


    Private m_freq(-1) As Single
    Private m_phase(-1) As Single

    ' =====================================================================================
    '  PUBLIC FUNCTIONS
    ' =====================================================================================
    Friend Enum WaveTypes As Int32
        Sine = 0
        Triangle = 1
        Square = 2
        Ramp = 3
        'RampNeg = 4
    End Enum

    Friend Sub SetWaveType(ByVal wavetype As WaveTypes)
        If wavetype <> m_WaveType Then
            m_WaveType = wavetype
            InitWaveTable()
        End If
    End Sub

    Friend Sub SetFreq(ByVal fmin As Double, ByVal fmax As Double)
        m_Fmin = CSng(fmin)
        m_Fmax = CSng(fmax)
        InitFrequencyTable()
    End Sub

    Friend Sub SetVolumeOut(ByVal v As Int32)
        m_OutVolume = Short.MaxValue * CSng(Math.Pow(v * 0.01F, 2))
    End Sub

    Friend Sub SetVolumeDrive(ByVal v As Int32)
        m_DriveVolume = 0.01F * CSng(Math.Pow(v * 0.01F, 2))
        InitVolumeTable()
    End Sub

    Friend Sub PlayStart(ByVal OutputDeviceIndex As Int32)
        If OutputDeviceIndex <> m_OutputDeviceIndex Then
            PlayStop()
            m_OutputDeviceIndex = OutputDeviceIndex
        End If
        If AOUT Is Nothing Then Return
        If m_Playing Then Return
        InitWaveTable()
        AOUT.OutputOn(m_OutputDeviceIndex, _
                    m_SamplesPerSec, _
                    m_Channels, _
                    m_LenBuffers, _
                    m_NumBuffers, _
                    AddressOf FillBuffer)
        m_Playing = True
    End Sub

    Friend Sub PlayStop()
        If AOUT Is Nothing Then Return
        If Not m_Playing Then Return
        Threading.Thread.Sleep(AOUT.GetTotalDelayMillisec() + 20)
        'Debug.Print(ao.GetTotalDelayMillisec().ToString)
        AOUT.OutputOff()
        m_Playing = False
    End Sub


    ' =====================================================================================
    '  INIT WAVETABLE
    ' =====================================================================================
    Private Sub InitWaveTable()
        If m_SamplesPerSec = 0 Then Return
        If m_WaveTable Is Nothing OrElse m_SamplesPerSec <> m_WaveTable.Length Then
            ReDim m_WaveTable(m_SamplesPerSec - 1)
        End If
        Dim i As Int32
        Select Case m_WaveType
            Case WaveTypes.Sine
                Dim k_pi As Double = (Math.PI * 2) / m_WaveTable.Length
                For i = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(Math.Sin(i * k_pi))
                Next
            Case WaveTypes.Triangle
                For i = 0 To m_WaveTable.Length \ 4 - 1
                    m_WaveTable(i) = CSng(4 * i / m_WaveTable.Length)
                Next
                For i = m_WaveTable.Length \ 4 To m_WaveTable.Length * 3 \ 4 - 1
                    m_WaveTable(i) = CSng(2 - (4 * i / m_WaveTable.Length))
                Next
                For i = m_WaveTable.Length * 3 \ 4 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(4 * i / m_WaveTable.Length - 4)
                Next
            Case WaveTypes.Square
                For i = 0 To m_WaveTable.Length \ 2 - 1
                    m_WaveTable(i) = 1
                Next
                For i = m_WaveTable.Length \ 2 To m_WaveTable.Length - 1
                    m_WaveTable(i) = -1
                Next
            Case WaveTypes.Ramp
                For i = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = -CSng(2 * (1 - i / m_WaveTable.Length) - 1)
                Next
                'Case WaveTypes.RampNeg
                '    For i = 0 To m_WaveTable.Length - 1
                '        m_WaveTable(i) = -CSng(2 * i / m_WaveTable.Length - 1)
                '    Next
        End Select
    End Sub


    ' =====================================================================================
    '  INIT
    ' =====================================================================================
    Friend Sub InitVolumeTable()
        ReDim m_vol1(BmpW - 1)
        ReDim m_vol2(BmpW - 1)
        For i As Int32 = 0 To BmpW - 1
            Dim k As Single = CSng(i / (BmpW - 1))
            ' ------------------------------------------------ normal
            m_vol1(i) = k
            m_vol2(i) = 1 - k
            ' ------------------------------------------------ expanded
            'm_vol1(i) = -0.5F + 2.0F * k
            'm_vol2(i) = 1.0F - (-0.5F + 2.0F * k)
        Next
        For i As Int32 = 0 To BmpW - 1
            m_vol1(i) *= m_DriveVolume
            m_vol2(i) *= m_DriveVolume
        Next
    End Sub

    Private Sub InitFrequencyTable()
        ReDim Preserve m_freq(BmpH - 1)
        ReDim Preserve m_phase(BmpH - 1)
        For i As Int32 = 0 To BmpH - 1
            m_freq(i) = (m_Fmin + (BmpH - 1 - i) * ((m_Fmax - m_Fmin) / BmpH))
        Next
    End Sub

    ' --------------------------------------------------------------------- Loop vars
    Private Kred As Single = 0.299F
    Private Kgreen As Single = 0.587F
    Private Kblue As Single = 0.114F

    Private modulo As Single
    Private wave As Single

    Private r As Byte
    Private g As Byte
    Private b As Byte
    Private gray As Single
    Private ptr As Int32


    Private left As Single
    Private right As Single

    ' =====================================================================================
    '  FILL BUFFER
    ' =====================================================================================
    Private Sub FillBuffer(ByRef data() As Int16)

        If BmpBytes.Length < BmpW * BmpH * 3 Then Return

        Dim m_SampleTime As Double = 1 / m_SamplesPerSec

        modulo = m_WaveTable.Length - 1

        For i As Int32 = 0 To data.Length - 1 Step 2

            left = 0
            right = 0
            ' --------------------------------------------------------------------- Loop for each pixel 
            For y As Int32 = 0 To BmpH - 1

                m_phase(y) = (m_phase(y) + m_freq(y)) Mod modulo
                wave = m_WaveTable(CInt(m_phase(y)))

                ptr = y * BmpStride

                For x As Int32 = 0 To BmpW - 1
                    ' ------------------------------------------------------------- R/G/B to Gray
                    'r = BmpBytes(disp + 2)
                    'g = BmpBytes(disp + 1)
                    'b = BmpBytes(disp)
                    'gray = r * Kred + g * Kgreen + b * Kblue
                    'disp += 3
                    ' ------------------------------------------------------------- Green to Gray
                    gray = BmpBytes(ptr + 1)
                    ptr += 3
                    ' ------------------------------------------------------------- 
                    left += m_vol1(x) * gray * wave
                    right += m_vol2(x) * gray * wave
                Next
            Next
            ' ---------------------------------------------------------- hard limit. +1/-1
            If (left > 1.0F) Then left = 1.0F
            If (left < -1.0F) Then left = -1.0F
            If (right > 1.0F) Then right = 1.0F
            If (right < -1.0F) Then right = -1.0F
            ' ---------------------------------------------------------- hard limit. +1/-1
            'right = 0.5F * (Math.Abs(right + 1.0F) - Math.Abs(right - 1.0F))
            'left = 0.5F * (Math.Abs(left + 1.0F) - Math.Abs(left - 1.0F))
            ' ---------------------------------------------------------- soft saturation
            'left = 1.5F * left - 0.5F * left * left * left
            'right = 1.5F * right - 0.5F * right * right * right
            ' ---------------------------------------------------------- 
            data(i) = CShort(right * m_OutVolume)
            data(i + 1) = CShort(left * m_OutVolume)
        Next
    End Sub


End Class

