﻿
Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend VideoInDevice As String = ""

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                               ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                               ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("FormMain_Top", r.Top))
            f.WriteLine(TabString("FormMain_Left", r.Left))
            f.WriteLine(TabString("FormMain_Width", r.Width))
            f.WriteLine(TabString("FormMain_Height", r.Height))
            f.WriteLine(TabString("FormMain_WindowState", Form1.WindowState))
            '
            r = GetFormRectangle(Form_VideoInControls)
            f.WriteLine(TabString("Form_VideoInControls_Top", r.Top))
            f.WriteLine(TabString("Form_VideoInControls_Left", r.Left))
            f.WriteLine(TabString("Form_VideoInControls_VisibleAtStart", Form_VideoInControls.Visible And Form_VideoInControls.WindowState <> FormWindowState.Minimized))
            '
            f.WriteLine("")
            f.WriteLine(TabString("==========================================="))
            f.WriteLine(TabString("Language", Language))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Video Input Params"))
            f.WriteLine(TabString("==========================================="))
            f.WriteLine(TabString("VideoInDevice", VideoInDevice))
            f.WriteLine(TabString("VideoFormat", VideoFormatParams.VideoFormat))
            f.WriteLine(TabString("VideoSize", VideoFormatParams.VideoSize))
            f.WriteLine(TabString("VideoFPS", VideoFormatParams.VideoFPS))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Params"))
            f.WriteLine(TabString("==========================================="))
            f.WriteLine(TabString("FlipX", Form1.chk_FlipX.Checked.ToString))
            f.WriteLine(TabString("FlipY", Form1.chk_FlipY.Checked.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Audio InOut")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SelectedAudioOut", Form1.SelectedAudioOut))
            '
            f.WriteLine("")
            f.WriteLine(" Generators")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("AudioEnabled", Form1.chk_AudioEnabled.Checked.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Sensorial")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("WaveType", Form1.cmb_WaveType.Text))
            f.WriteLine(TabString("Fmin", Form1.tbar_FreqMin.Value.ToString))
            f.WriteLine(TabString("Fmax", Form1.tbar_FreqMax.Value.ToString))
            f.WriteLine(TabString("VolumeOut", Form1.tbar_VolumeOut.Value.ToString))
            f.WriteLine(TabString("VolumeDrive", Form1.tbar_VolumeDrive.Value.ToString))
            f.WriteLine(TabString("MouseTest", Form1.chk_MouseTest.Checked.ToString))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI()
        ' ------------------------------------------------------------------------------- defaults
        VideoInDevice = ""
        VideoFormatParams.VideoFormat = "RGB42"
        VideoFormatParams.VideoSize = "320 x 240"
        VideoFormatParams.VideoFPS = "30"
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)
            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "FormMain_Top" : Form1.Top = Val_Int(l)
                    Case "FormMain_Left" : Form1.Left = Val_Int(l)
                    Case "FormMain_Width" : Form1.Width = Val_Int(l)
                    Case "FormMain_Height" : Form1.Height = Val_Int(l)
                    Case "FormMain_WindowState" : Form1.WindowState = CType(Val(l), FormWindowState)
                        ' ------------------------------------------------------------------------------ 
                    Case "Form_VideoInControls_Top" : Form_VideoInControls.Top = Val_Int(l)
                    Case "Form_VideoInControls_Left" : Form_VideoInControls.Left = Val_Int(l)
                    Case "Form_VideoInControls_VisibleAtStart" : Form_VideoInControls_VisibleAtStart = l = "True"
                        ' ------------------------------------------------------------------------------ Video in device
                    Case "VideoInDevice" : VideoInDevice = l
                    Case "VideoFormat" : VideoFormatParams.VideoFormat = l
                    Case "VideoSize" : VideoFormatParams.VideoSize = l
                    Case "VideoFPS" : VideoFormatParams.VideoFPS = l
                        ' ------------------------------------------------------------------------------
                    Case "Language" : Language = l
                        ' ------------------------------------------------------------------------------
                    Case "FlipX" : Form1.chk_FlipX.Checked = l = "True"
                    Case "FlipY" : Form1.chk_FlipY.Checked = l = "True"
                        ' ----------------------------------------------------------------- 
                    Case "SelectedAudioOut" : Form1.SelectedAudioOut = Val_Int(l)
                        ' -----------------------------------------------------------------
                    Case "AudioEnabled" : Form1.chk_AudioEnabled.Checked = l = "True"
                        ' -----------------------------------------------------------------
                    Case "WaveType" : Form1.cmb_WaveType.Text = l
                    Case "Fmin" : Form1.tbar_FreqMin.Value = Val_Int(l)
                    Case "Fmax" : Form1.tbar_FreqMax.Value = Val_Int(l)
                    Case "VolumeOut" : Form1.tbar_VolumeOut.Value = Val_Int(l)
                    Case "VolumeDrive" : Form1.tbar_VolumeDrive.Value = Val_Int(l)
                    Case "MouseTest" : Form1.chk_MouseTest.Checked = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
        LimitFormPosition(Form_VideoInControls)
        '
    End Sub

    'Private Sub PrepareSaveLoadFolder()
    '    If My.Computer.FileSystem.FileExists(FileName) Then
    '        SaveLoadFolder = File_GetPath(FileName)
    '    End If
    '    If Not FolderExists(SaveLoadFolder) Then
    '        SaveLoadFolder = Application.StartupPath
    '    End If
    'End Sub

    Public Function File_GetPath(ByVal str As String) As String
        Try
            Return IO.Path.GetDirectoryName(str) & "\"
        Catch
            Return str
        End Try
    End Function


End Module
